@@ -102,7 +102,7 @@ expand_token (struct obstack *obs, read_type expansion, token_type t,
          If another character is found, this string is not a
          macro.   */
 
-      if (IS_ALPHA (*text) || (*text == '/' && IS_ALPHA (*(text+1))))
+      if (IS_ALPHA (*text))
         {
           sym = lookup_symbol (text, SYMBOL_LOOKUP);
           if (sym == NULL || SYMBOL_TYPE (sym) == TOKEN_VOID)
@@ -115,6 +115,17 @@ expand_token (struct obstack *obs, read_type expansion, token_type t,
           else
             expand_macro (sym, expansion);
         }
+      else if (*text == '/' && IS_ALPHA (*(text+1)))
+        {
+          sym = lookup_symbol (text+1, SYMBOL_LOOKUP);
+          if (sym != NULL)
+            {
+              MP4HERROR ((warning_status, 0,
+                _("Warning:%s:%d: `<%s>' found without begin tag"),
+                    CURRENT_FILE_LINE, text));
+            }
+          expand_unknown_tag (text, expansion);
+        }
       else
         shipout_text (obs, TOKEN_DATA_TEXT (td));
       break;
@@ -135,9 +146,13 @@ expand_token (struct obstack *obs, read_type expansion, token_type t,
 | whether the argument read are the last for the active macro call.  The   |
 | argument read are the last for the active macro call.  The argument are  |
 | on the obstack OBS, indirectly through expand_token ().                  |
+| On exit, this function returns                                           |
+|      0: close bracket found                                              |
+|      1: last argument                                                    |
+|      2: other arguments follow                                           |
 `-------------------------------------------------------------------------*/
 
-static boolean
+static int
 expand_argument (struct obstack *obs, read_type expansion, token_data *argp,
                  char *last_char_ptr)
 {
@@ -145,6 +160,7 @@ expand_argument (struct obstack *obs, read_type expansion, token_data *argp,
   token_type t;
   token_data td;
   int group_level = 0;
+  int rc;
   boolean in_string = FALSE;
 
   *last_char_ptr = ' ';
@@ -164,9 +180,9 @@ expand_argument (struct obstack *obs, read_type expansion, token_data *argp,
       t = next_token (&td, expansion, FALSE);
     }
 
+  rc = 0;
   while (1)
     {
-
       switch (t)
         {                       /* TOKSW */
         case TOKEN_SPACE:
@@ -191,7 +207,9 @@ expand_argument (struct obstack *obs, read_type expansion, token_data *argp,
                   TOKEN_DATA_TYPE (argp) = TOKEN_TEXT;
                   TOKEN_DATA_TEXT (argp) = obstack_finish (obs);
                 }
-              return (boolean) (IS_SPACE(*TOKEN_DATA_TEXT (&td)));
+              if (IS_SPACE(*TOKEN_DATA_TEXT (&td)))
+                rc = 2;
+              return rc;
             }
           expand_token (obs, expansion, t, &td);
           break;
@@ -250,6 +268,7 @@ expand_argument (struct obstack *obs, read_type expansion, token_data *argp,
 
       *last_char_ptr = LAST_CHAR (TOKEN_DATA_TEXT (&td));
       t = next_token (&td, expansion, in_string);
+      rc = 1;
     }
 }
 
@@ -269,11 +288,11 @@ collect_arguments (char *symbol_name, read_type expansion,
   token_data td;
   token_data *tdp;
   char *last_addr;
-  boolean more_args;
+  int more_args;
   char last_char = ' ';
 
   TOKEN_DATA_TYPE (&td) = TOKEN_TEXT;
-  TOKEN_DATA_TEXT (&td) = symbol_name;
+  TOKEN_DATA_TEXT (&td) = xstrdup (symbol_name);
   tdp = (token_data *) obstack_copy (arguments, (voidstar) &td, sizeof (td));
   obstack_grow (argptr, (voidstar) &tdp, sizeof (tdp));
 
@@ -299,12 +318,11 @@ collect_arguments (char *symbol_name, read_type expansion,
             obstack_copy (arguments, (voidstar) &td, sizeof (td));
           obstack_grow (argptr, (voidstar) &tdp, sizeof (tdp));
         }
-      while (more_args);
+      while (more_args == 2);
 
       /*  If the last argument is empty, it is removed.  We need it to
           remove white spaces before closing brackets.  */
-      if (TOKEN_DATA_TYPE (tdp) == TOKEN_TEXT
-          && strlen (TOKEN_DATA_TEXT (tdp)) == 0)
+      if (more_args == 0)
         argptr->next_free = last_addr;
     }
   else
@@ -504,6 +522,7 @@ expand_macro (symbol *sym, read_type expansion)
 {
   struct obstack arguments, argptr, body;
   token_data **argv;
+  token_data *td;
   int argc, i;
   struct obstack *obs_expansion;
   const char *expanded;
@@ -543,19 +562,29 @@ expand_macro (symbol *sym, read_type expansion)
                              &arguments);
   argc = obstack_object_size (&argptr) / sizeof (token_data *);
 
-  if (SYMBOL_CONTAINER (sym))
+  if (SYMBOL_CONTAINER (sym) && !slash)
     {
       collect_body (SYMBOL_NAME (sym), READ_BODY, &argptr, &body);
       argv = (token_data **) obstack_finish (&argptr);
     }
   else
     {
+      if (SYMBOL_CONTAINER (sym))
+        {
+          TOKEN_DATA_TYPE (td) = TOKEN_TEXT;
+          TOKEN_DATA_TEXT (td) = xstrdup ("");
+          obstack_grow (&argptr, (voidstar) &td, sizeof (td));
+        }
+
       argv = (token_data **) obstack_finish (&argptr);
       if (slash)
         {
           cp = TOKEN_DATA_TEXT (argv[argc-1]);
           if (IS_SLASH(*cp) && *(cp+1) == '\0')
-            argc--;
+            {
+              *cp = '\0';
+              argc--;
+            }
           else if (IS_SLASH (LAST_CHAR (cp)))
             {
               if (IS_SPACE (*(cp+strlen(cp)-2)))
@@ -592,13 +621,13 @@ expand_macro (symbol *sym, read_type expansion)
         {
           shipout_string (obs_expansion, TOKEN_DATA_TEXT (argv[i]), 0);
         }
-      if (!SYMBOL_CONTAINER (sym) && slash)
+      if (slash)
         {
           obstack_1grow (obs_expansion, CHAR_SLASH);
         }
 
       obstack_1grow (obs_expansion, '>');
-      if (SYMBOL_CONTAINER (sym))
+      if (SYMBOL_CONTAINER (sym) && !slash)
         {
           shipout_string (obs_expansion, TOKEN_DATA_TEXT (argv[argc]), 0);
           obstack_grow (obs_expansion, "</", 2);
@@ -707,6 +736,8 @@ expand_unknown_tag (char *name, read_type expansion)
 
   if (slash)
     {
+      if (! (exp_flags & EXP_NOSPACE_BSLASH))
+        obstack_1grow (obs_expansion, ' ');
       obstack_1grow (obs_expansion, CHAR_SLASH);
     }
   obstack_1grow (obs_expansion, '>');
