@@ -278,7 +278,7 @@ make_room_for (int length)
 | IMPORTANT WARNING: string must be null-terminated.  |
 `----------------------------------------------------*/
 
-int
+void
 remove_special_chars (char *s, boolean restore_quotes)
 {
   int offset = 0;
@@ -289,8 +289,7 @@ remove_special_chars (char *s, boolean restore_quotes)
       After that, string is shifyed.  */
   if (restore_quotes)
     {
-      for (cp=s; *cp != '\0' && *cp != CHAR_LQUOTE && *cp != CHAR_RQUOTE
-             && *cp != CHAR_BGROUP && *cp != CHAR_EGROUP; cp++)
+      for (cp=s; *cp != '\0' && !(IS_GROUP (*cp)); cp++)
         {
           if (*cp == CHAR_QUOTE)
             *cp = '"';
@@ -298,18 +297,16 @@ remove_special_chars (char *s, boolean restore_quotes)
     }
   else
     {
-      for (cp=s; *cp != '\0' && *cp != CHAR_LQUOTE && *cp != CHAR_RQUOTE
-             && *cp != CHAR_BGROUP && *cp != CHAR_EGROUP; cp++)
+      for (cp=s; *cp != '\0' && !(IS_GROUP (*cp)); cp++)
         ;
     }
 
   if (*cp == '\0')
-    return offset;
+    return;
 
   for ( ; *cp != '\0'; cp++)
     {
-      if (*cp == CHAR_LQUOTE || *cp == CHAR_RQUOTE
-       || *cp == CHAR_BGROUP || *cp == CHAR_EGROUP)
+      if (IS_GROUP (*cp))
         offset++;
       else if ((*cp == CHAR_QUOTE) && restore_quotes)
         *(cp-offset) = '"';
@@ -317,19 +314,18 @@ remove_special_chars (char *s, boolean restore_quotes)
         *(cp-offset) = *cp;
     }
   *(cp-offset) = '\0';
-  return offset;
 }
 
 /*----------------------------------------------------.
 | Remove leading and trailing stars in tag names.     |
+| Remove trailing slash in tag attributes.            |
 `----------------------------------------------------*/
 
 static void
-remove_stars (char *s, int *length_ptr)
+remove_stars_and_slash (char *s)
 {
-  int offset, i;
+  int offset;
   register char *cp;
-  int length = *length_ptr;
 
   offset = 0;
   cp = s;
@@ -340,15 +336,26 @@ remove_stars (char *s, int *length_ptr)
       cp++;
     }
 
-  for (i=0; i<length; cp++, i++)
+  for ( ; *cp != '\0'; cp++)
     {
-      *(cp-offset) = *cp;
-      if (*cp == '<' && i<length-1)
+      if (*cp == CHAR_SLASH)
         {
+          if (exp_flags & EXP_REMOVE_TRAILING_SLASH)
+            {
+              offset++;
+              if (cp > s && IS_SPACE(*(cp-1)))
+                offset++;
+            }
+          else
+            *(cp-offset) = '/';
+        }
+      else if (*cp == '<')
+        {
+          *(cp-offset) = *cp;
           if (*(cp+1) == '*' && !(exp_flags & EXP_LEAVE_LEADING_STAR))
             {
               offset++;
-              cp++, i++;
+              cp++;
               continue;
             }
           if (exp_flags & EXP_LEAVE_TRAILING_STAR)
@@ -356,27 +363,27 @@ remove_stars (char *s, int *length_ptr)
 
           if (*(cp+1) == '/')
             {
-              cp++, i++;
+              cp++;
               *(cp-offset) = *cp;
             }
-          cp++, i++;
-          if (i<length && IS_ALPHA (*cp))
+          cp++;
+          if (IS_ALPHA (*cp))
             {
-              while (i<length && IS_ALNUM(*cp))
+              while (IS_ALNUM(*cp))
                 {
                   *(cp-offset) = *cp;
-                  cp++, i++;
+                  cp++;
                 }
-              if (i<length && *cp == '*' )
+              if (*cp == '*' )
                 offset++;
               else
-                {
-                  cp--, i--;
-                }
+                cp--;
             }
         }
+      else
+        *(cp-offset) = *cp;
     }
-  *length_ptr -= offset;
+  *(cp-offset) = '\0';
 }
 
 /*------------------------------------------------------------------------.
@@ -446,17 +453,18 @@ output_text (const char *text, int length)
 `-------------------------------------------------------------------------*/
 
 void
-shipout_text (struct obstack *obs, char *text, int length)
+shipout_text (struct obstack *obs, char *text)
 {
   static boolean start_of_output_line = TRUE;
   char line[20];
   const char *cursor;
+  int length;
 
   /* If output goes to an obstack, merely add TEXT to it.  */
 
   if (obs != NULL)
     {
-      obstack_grow (obs, text, length);
+      obstack_grow (obs, text, strlen (text));
       return;
     }
 
@@ -466,14 +474,23 @@ shipout_text (struct obstack *obs, char *text, int length)
     return;
 
   /* Restitute some special characters  */
-  length -= remove_special_chars (text, TRUE);
+  remove_special_chars (text, TRUE);
 
-  /* Remove leading and trailing stars in tag names  */
-  if (!(exp_flags & (EXP_LEAVE_TRAILING_STAR | EXP_LEAVE_TRAILING_STAR)))
-    remove_stars (text, &length);
+  /* Remove leading and trailing stars in tag names
+     and trailing slash in attributes */
+  if (!(exp_flags & (EXP_LEAVE_TRAILING_STAR | EXP_LEAVE_TRAILING_STAR))
+      || (exp_flags & EXP_REMOVE_TRAILING_SLASH))
+    remove_stars_and_slash (text);
+
+  if (*text == '\0')
+    {
+      after_left_angle = FALSE;
+      return;
+    }
 
   /* Output TEXT to a file, or in-memory diversion buffer.  */
 
+  length = strlen (text);
   if (!sync_output)
     switch (length)
       {
@@ -497,7 +514,7 @@ shipout_text (struct obstack *obs, char *text, int length)
         text += length - 1;
       }
   else
-    for (; length-- > 0; text++)
+    for (; *(text+1) != '\0'; text++)
       {
         if (start_of_output_line)
           {
